@@ -1,6 +1,6 @@
 /***
  * ASM examples: examples showing how ASM can be used
- * Copyright (c) 2000-2005 INRIA, France Telecom
+ * Copyright (c) 2000-2007 INRIA, France Telecom
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,7 +37,7 @@ public class ArraySet {
 
     private int size;
 
-    public boolean contains(int v) {
+    public boolean contains(final int v) {
         for (int i = 0; i < size; ++i) {
             if (values[i] == v) {
                 return true;
@@ -46,21 +46,19 @@ public class ArraySet {
         return false;
     }
 
-    public void add(int v) {
+    public void add(final int v) {
         if (!contains(v)) {
             if (size == values.length) {
                 System.err.println("[enlarge]");
                 int[] newValues = new int[values.length + 3];
-                for (int i = 0; i < size; ++i) {
-                    newValues[i] = values[i];
-                }
+                System.arraycopy(values, 0, newValues, 0, size);
                 values = newValues;
             }
             values[size++] = v;
         }
     }
 
-    public void remove(int v) {
+    public void remove(final int v) {
         int i = 0;
         int j = 0;
         while (i < size) {
@@ -75,7 +73,7 @@ public class ArraySet {
 
     // test method
 
-    public static void main(String[] args) {
+    public static void main(final String[] args) {
         ArraySet s = new ArraySet();
         System.err.println("add 1");
         s.add(1);
