@@ -65,6 +65,12 @@ public class ClassWriter implements ClassVisitor {
      */
     public static final int COMPUTE_FRAMES = 2;
 
+    /**
+     * Pseudo access flag to distinguish between the synthetic attribute and
+     * the synthetic access flag.
+     */
+    static final int ACC_SYNTHETIC_ATTRIBUTE = 0x40000;
+    
     /**
      * The type of instructions without any argument.
      */
@@ -566,7 +572,10 @@ public class ClassWriter implements ClassVisitor {
      *        original class and also to copy other fragments of original
      *        bytecode where applicable.
      * @param flags option flags that can be used to modify the default behavior
-     *        of this class. See {@link #COMPUTE_MAXS}, {@link #COMPUTE_FRAMES}.
+     *        of this class. <i>These option flags do not affect methods that
+     *        are copied as is in the new class. This means that the maximum
+     *        stack size nor the stack frames will be computed for these
+     *        methods</i>. See {@link #COMPUTE_MAXS}, {@link #COMPUTE_FRAMES}.
      */
     public ClassWriter(final ClassReader classReader, final int flags) {
         this(flags);
@@ -705,6 +714,9 @@ public class ClassWriter implements ClassVisitor {
      * @return the bytecode of the class that was build with this class writer.
      */
     public byte[] toByteArray() {
+        if (index > Short.MAX_VALUE) {
+            throw new RuntimeException("Class file too large!");
+        }
         // computes the real size of the bytecode of this class
         int size = 24 + 2 * interfaceCount;
         int nbFields = 0;
@@ -748,7 +760,7 @@ public class ClassWriter implements ClassVisitor {
             newUTF8("Deprecated");
         }
         if ((access & Opcodes.ACC_SYNTHETIC) != 0
-                && (version & 0xffff) < Opcodes.V1_5)
+                && ((version & 0xFFFF) < Opcodes.V1_5 || (access & ACC_SYNTHETIC_ATTRIBUTE) != 0))
         {
             ++attributeCount;
             size += 6;
@@ -779,7 +791,10 @@ public class ClassWriter implements ClassVisitor {
         ByteVector out = new ByteVector(size);
         out.putInt(0xCAFEBABE).putInt(version);
         out.putShort(index).putByteArray(pool.data, 0, pool.length);
-        out.putShort(access).putShort(name).putShort(superName);
+        int mask = Opcodes.ACC_DEPRECATED
+                | ClassWriter.ACC_SYNTHETIC_ATTRIBUTE
+                | ((access & ClassWriter.ACC_SYNTHETIC_ATTRIBUTE) / (ClassWriter.ACC_SYNTHETIC_ATTRIBUTE / Opcodes.ACC_SYNTHETIC));
+        out.putShort(access & ~mask).putShort(name).putShort(superName);
         out.putShort(interfaceCount);
         for (int i = 0; i < interfaceCount; ++i) {
             out.putShort(interfaces[i]);
@@ -816,7 +831,7 @@ public class ClassWriter implements ClassVisitor {
             out.putShort(newUTF8("Deprecated")).putInt(0);
         }
         if ((access & Opcodes.ACC_SYNTHETIC) != 0
-                && (version & 0xffff) < Opcodes.V1_5)
+                && ((version & 0xFFFF) < Opcodes.V1_5 || (access & ACC_SYNTHETIC_ATTRIBUTE) != 0))
         {
             out.putShort(newUTF8("Synthetic")).putInt(0);
         }
@@ -1098,8 +1113,8 @@ public class ClassWriter implements ClassVisitor {
         if (result == null) {
             pool.putByte(LONG).putLong(value);
             result = new Item(index, key);
-            put(result);
             index += 2;
+            put(result);
         }
         return result;
     }
@@ -1117,8 +1132,8 @@ public class ClassWriter implements ClassVisitor {
         if (result == null) {
             pool.putByte(DOUBLE).putLong(key.longVal);
             result = new Item(index, key);
-            put(result);
             index += 2;
+            put(result);
         }
         return result;
     }
@@ -1322,7 +1337,7 @@ public class ClassWriter implements ClassVisitor {
      * @param i the item to be added to the constant pool's hash table.
      */
     private void put(final Item i) {
-        if (index > threshold) {
+        if (index + typeCount > threshold) {
             int ll = items.length;
             int nl = ll * 2 + 1;
             Item[] newItems = new Item[nl];
