@@ -1,6 +1,6 @@
 /***
  * ASM: a very small and fast Java bytecode manipulation framework
- * Copyright (c) 2000-2005 INRIA, France Telecom
+ * Copyright (c) 2000-2007 INRIA, France Telecom
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,17 +40,26 @@ final class Item {
     /**
      * Index of this item in the constant pool.
      */
-    short index;
+    int index;
 
     /**
      * Type of this constant pool item. A single class is used to represent all
      * constant pool item types, in order to minimize the bytecode size of this
-     * package. The value of this field is I, J, F, D, S, s, C, T, G, M, or N
-     * (for Constant Integer, Long, Float, Double, STR, UTF8, Class, NameType,
-     * Fieldref, Methodref, or InterfaceMethodref constant pool items
-     * respectively).
+     * package. The value of this field is one of {@link ClassWriter#INT},
+     * {@link ClassWriter#LONG}, {@link ClassWriter#FLOAT},
+     * {@link ClassWriter#DOUBLE}, {@link ClassWriter#UTF8},
+     * {@link ClassWriter#STR}, {@link ClassWriter#CLASS},
+     * {@link ClassWriter#NAME_TYPE}, {@link ClassWriter#FIELD},
+     * {@link ClassWriter#METH}, {@link ClassWriter#IMETH}.
+     * 
+     * Special Item types are used for Items that are stored in the ClassWriter
+     * {@link ClassWriter#typeTable}, instead of the constant pool, in order to
+     * avoid clashes with normal constant pool items in the ClassWriter constant
+     * pool's hash table. These special item types are
+     * {@link ClassWriter#TYPE_NORMAL}, {@link ClassWriter#TYPE_UNINIT} and
+     * {@link ClassWriter#TYPE_MERGED}.
      */
-    char type;
+    int type;
 
     /**
      * Value of this item, for an integer item.
@@ -62,16 +71,6 @@ final class Item {
      */
     long longVal;
 
-    /**
-     * Value of this item, for a float item.
-     */
-    float floatVal;
-
-    /**
-     * Value of this item, for a double item.
-     */
-    double doubleVal;
-
     /**
      * First part of the value of this item, for items that do not hold a
      * primitive value.
@@ -107,19 +106,27 @@ final class Item {
     Item() {
     }
 
+    /**
+     * Constructs an uninitialized {@link Item} for constant pool element at
+     * given position.
+     * 
+     * @param index index of the item to be constructed.
+     */
+    Item(final int index) {
+        this.index = index;
+    }
+
     /**
      * Constructs a copy of the given item.
      * 
      * @param index index of the item to be constructed.
      * @param i the item that must be copied into the item to be constructed.
      */
-    Item(final short index, final Item i) {
+    Item(final int index, final Item i) {
         this.index = index;
         type = i.type;
         intVal = i.intVal;
         longVal = i.longVal;
-        floatVal = i.floatVal;
-        doubleVal = i.doubleVal;
         strVal1 = i.strVal1;
         strVal2 = i.strVal2;
         strVal3 = i.strVal3;
@@ -132,7 +139,7 @@ final class Item {
      * @param intVal the value of this item.
      */
     void set(final int intVal) {
-        this.type = 'I';
+        this.type = ClassWriter.INT;
         this.intVal = intVal;
         this.hashCode = 0x7FFFFFFF & (type + intVal);
     }
@@ -143,7 +150,7 @@ final class Item {
      * @param longVal the value of this item.
      */
     void set(final long longVal) {
-        this.type = 'J';
+        this.type = ClassWriter.LONG;
         this.longVal = longVal;
         this.hashCode = 0x7FFFFFFF & (type + (int) longVal);
     }
@@ -154,8 +161,8 @@ final class Item {
      * @param floatVal the value of this item.
      */
     void set(final float floatVal) {
-        this.type = 'F';
-        this.floatVal = floatVal;
+        this.type = ClassWriter.FLOAT;
+        this.intVal = Float.floatToRawIntBits(floatVal);
         this.hashCode = 0x7FFFFFFF & (type + (int) floatVal);
     }
 
@@ -165,8 +172,8 @@ final class Item {
      * @param doubleVal the value of this item.
      */
     void set(final double doubleVal) {
-        this.type = 'D';
-        this.doubleVal = doubleVal;
+        this.type = ClassWriter.DOUBLE;
+        this.longVal = Double.doubleToRawLongBits(doubleVal);
         this.hashCode = 0x7FFFFFFF & (type + (int) doubleVal);
     }
 
@@ -179,7 +186,7 @@ final class Item {
      * @param strVal3 third part of the value of this item.
      */
     void set(
-        final char type,
+        final int type,
         final String strVal1,
         final String strVal2,
         final String strVal3)
@@ -189,18 +196,19 @@ final class Item {
         this.strVal2 = strVal2;
         this.strVal3 = strVal3;
         switch (type) {
-            case 's':
-            case 'S':
-            case 'C':
+            case ClassWriter.UTF8:
+            case ClassWriter.STR:
+            case ClassWriter.CLASS:
+            case ClassWriter.TYPE_NORMAL:
                 hashCode = 0x7FFFFFFF & (type + strVal1.hashCode());
                 return;
-            case 'T':
+            case ClassWriter.NAME_TYPE:
                 hashCode = 0x7FFFFFFF & (type + strVal1.hashCode()
                         * strVal2.hashCode());
                 return;
-            // case 'G':
-            // case 'M':
-            // case 'N':
+                // ClassWriter.FIELD:
+                // ClassWriter.METH:
+                // ClassWriter.IMETH:
             default:
                 hashCode = 0x7FFFFFFF & (type + strVal1.hashCode()
                         * strVal2.hashCode() * strVal3.hashCode());
@@ -208,39 +216,39 @@ final class Item {
     }
 
     /**
-     * Indicates if the given item is equal to this one.
+     * Indicates if the given item is equal to this one. <i>This method assumes
+     * that the two items have the same {@link #type}</i>. 
      * 
-     * @param i the item to be compared to this one.
+     * @param i the item to be compared to this one. Both items must have the
+     *       same {@link #type}.
      * @return <tt>true</tt> if the given item if equal to this one,
      *         <tt>false</tt> otherwise.
      */
     boolean isEqualTo(final Item i) {
-        if (i.type == type) {
-            switch (type) {
-                case 'I':
-                    return i.intVal == intVal;
-                case 'J':
-                    return i.longVal == longVal;
-                case 'F':
-                    return i.floatVal == floatVal;
-                case 'D':
-                    return i.doubleVal == doubleVal;
-                case 's':
-                case 'S':
-                case 'C':
-                    return i.strVal1.equals(strVal1);
-                case 'T':
-                    return i.strVal1.equals(strVal1)
-                            && i.strVal2.equals(strVal2);
-                // case 'G':
-                // case 'M':
-                // case 'N':
-                default:
-                    return i.strVal1.equals(strVal1)
-                            && i.strVal2.equals(strVal2)
-                            && i.strVal3.equals(strVal3);
-            }
+        switch (type) {
+            case ClassWriter.UTF8:
+            case ClassWriter.STR:
+            case ClassWriter.CLASS:
+            case ClassWriter.TYPE_NORMAL:
+                return i.strVal1.equals(strVal1);
+            case ClassWriter.TYPE_MERGED:
+            case ClassWriter.LONG:
+            case ClassWriter.DOUBLE:
+                return i.longVal == longVal;
+            case ClassWriter.INT:
+            case ClassWriter.FLOAT:
+                return i.intVal == intVal;
+            case ClassWriter.TYPE_UNINIT:
+                return i.intVal == intVal && i.strVal1.equals(strVal1);
+            case ClassWriter.NAME_TYPE:
+                return i.strVal1.equals(strVal1) && i.strVal2.equals(strVal2);
+            // case ClassWriter.FIELD:
+            // case ClassWriter.METH:
+            // case ClassWriter.IMETH:
+            default:    
+                return i.strVal1.equals(strVal1) && i.strVal2.equals(strVal2)
+                        && i.strVal3.equals(strVal3);
         }
-        return false;
     }
+
 }
