@@ -1,6 +1,6 @@
 /***
  * ASM: a very small and fast Java bytecode manipulation framework
- * Copyright (c) 2000-2005 INRIA, France Telecom
+ * Copyright (c) 2000-2007 INRIA, France Telecom
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,12 +33,14 @@ package org.objectweb.asm;
  * A visitor to visit a Java method. The methods of this interface must be
  * called in the following order: [ <tt>visitAnnotationDefault</tt> ] (
  * <tt>visitAnnotation</tt> | <tt>visitParameterAnnotation</tt> |
- * <tt>visitAttribute</tt> )* [ <tt>visitCode</tt> ( <tt>visit</tt><i>X</i>Insn</tt> |
- * <tt>visitLabel</tt> | <tt>visitTryCatchBlock</tt> | <tt>visitLocalVariable</tt> |
- * <tt>visitLineNumber</tt>)* <tt>visitMaxs</tt> ] <tt>visitEnd</tt>. In
- * addition, the <tt>visit</tt><i>X</i>Insn</tt> and <tt>visitLabel</tt>
- * methods must be called in the sequential order of the bytecode instructions
- * of the visited code, and the <tt>visitLocalVariable</tt> and <tt>visitLineNumber</tt>
+ * <tt>visitAttribute</tt> )* [ <tt>visitCode</tt> ( <tt>visitFrame</tt> |
+ * <tt>visit</tt><i>X</i>Insn</tt> | <tt>visitLabel</tt> | <tt>visitTryCatchBlock</tt> |
+ * <tt>visitLocalVariable</tt> | <tt>visitLineNumber</tt>)* <tt>visitMaxs</tt> ]
+ * <tt>visitEnd</tt>. In addition, the <tt>visit</tt><i>X</i>Insn</tt>
+ * and <tt>visitLabel</tt> methods must be called in the sequential order of
+ * the bytecode instructions of the visited code, <tt>visitTryCatchBlock</tt>
+ * must be called <i>before</i> the labels passed as arguments have been
+ * visited, and the <tt>visitLocalVariable</tt> and <tt>visitLineNumber</tt>
  * methods must be called <i>after</i> the labels passed as arguments have been
  * visited.
  * 
@@ -53,11 +55,12 @@ public interface MethodVisitor {
     /**
      * Visits the default value of this annotation interface method.
      * 
-     * @return a non null visitor to the visit the actual default value of this
-     *         annotation interface method. The 'name' parameters passed to the
-     *         methods of this annotation visitor are ignored. Moreover, exacly
-     *         one visit method must be called on this annotation visitor,
-     *         followed by visitEnd.
+     * @return a visitor to the visit the actual default value of this
+     *         annotation interface method, or <tt>null</tt> if this visitor
+     *         is not interested in visiting this default value. The 'name'
+     *         parameters passed to the methods of this annotation visitor are
+     *         ignored. Moreover, exacly one visit method must be called on this
+     *         annotation visitor, followed by visitEnd.
      */
     AnnotationVisitor visitAnnotationDefault();
 
@@ -66,7 +69,8 @@ public interface MethodVisitor {
      * 
      * @param desc the class descriptor of the annotation class.
      * @param visible <tt>true</tt> if the annotation is visible at runtime.
-     * @return a non null visitor to visit the annotation values.
+     * @return a visitor to visit the annotation values, or <tt>null</tt> if
+     *         this visitor is not interested in visiting this annotation.
      */
     AnnotationVisitor visitAnnotation(String desc, boolean visible);
 
@@ -76,7 +80,8 @@ public interface MethodVisitor {
      * @param parameter the parameter index.
      * @param desc the class descriptor of the annotation class.
      * @param visible <tt>true</tt> if the annotation is visible at runtime.
-     * @return a non null visitor to visit the annotation values.
+     * @return a visitor to visit the annotation values, or <tt>null</tt> if
+     *         this visitor is not interested in visiting this annotation.
      */
     AnnotationVisitor visitParameterAnnotation(
         int parameter,
@@ -95,6 +100,64 @@ public interface MethodVisitor {
      */
     void visitCode();
 
+    /**
+     * Visits the current state of the local variables and operand stack
+     * elements. This method must(*) be called <i>just before</i> any
+     * instruction <b>i</b> that follows an unconditional branch instruction
+     * such as GOTO or THROW, that is the target of a jump instruction, or that
+     * starts an exception handler block. The visited types must describe the
+     * values of the local variables and of the operand stack elements <i>just
+     * before</i> <b>i</b> is executed. <br> <br> (*) this is mandatory only
+     * for classes whose version is greater than or equal to
+     * {@link Opcodes#V1_6 V1_6}. <br> <br> Packed frames are basically
+     * "deltas" from the state of the previous frame (very first frame is
+     * implicitly defined by the method's parameters and access flags): <ul>
+     * <li>{@link Opcodes#F_SAME} representing frame with exactly the same
+     * locals as the previous frame and with the empty stack.</li> <li>{@link Opcodes#F_SAME1}
+     * representing frame with exactly the same locals as the previous frame and
+     * with single value on the stack (<code>nStack</code> is 1 and
+     * <code>stack[0]</code> contains value for the type of the stack item).</li>
+     * <li>{@link Opcodes#F_APPEND} representing frame with current locals are
+     * the same as the locals in the previous frame, except that additional
+     * locals are defined (<code>nLocal</code> is 1, 2 or 3 and
+     * <code>local</code> elements contains values representing added types).</li>
+     * <li>{@link Opcodes#F_CHOP} representing frame with current locals are
+     * the same as the locals in the previous frame, except that the last 1-3
+     * locals are absent and with the empty stack (<code>nLocals</code> is 1,
+     * 2 or 3). </li> <li>{@link Opcodes#F_FULL} representing complete frame
+     * data.</li> </li> </ul>
+     * 
+     * @param type the type of this stack map frame. Must be
+     *        {@link Opcodes#F_NEW} for expanded frames, or
+     *        {@link Opcodes#F_FULL}, {@link Opcodes#F_APPEND},
+     *        {@link Opcodes#F_CHOP}, {@link Opcodes#F_SAME} or
+     *        {@link Opcodes#F_APPEND}, {@link Opcodes#F_SAME1} for compressed
+     *        frames.
+     * @param nLocal the number of local variables in the visited frame.
+     * @param local the local variable types in this frame. This array must not
+     *        be modified. Primitive types are represented by
+     *        {@link Opcodes#TOP}, {@link Opcodes#INTEGER},
+     *        {@link Opcodes#FLOAT}, {@link Opcodes#LONG},
+     *        {@link Opcodes#DOUBLE},{@link Opcodes#NULL} or
+     *        {@link Opcodes#UNINITIALIZED_THIS} (long and double are
+     *        represented by a single element). Reference types are represented
+     *        by String objects (representing internal names), and uninitialized 
+     *        types by Label objects (this label designates the NEW instruction 
+     *        that created this uninitialized value).
+     * @param nStack the number of operand stack elements in the visited frame.
+     * @param stack the operand stack types in this frame. This array must not
+     *        be modified. Its content has the same format as the "local" array.
+     * @throw IllegalStateException if a frame is visited just after another
+     *        one, without any instruction between the two (unless this frame
+     *        is a Opcodes#F_SAME frame, in which case it is silently ignored).
+     */
+    void visitFrame(
+        int type,
+        int nLocal,
+        Object[] local,
+        int nStack,
+        Object[] stack);
+
     // -------------------------------------------------------------------------
     // Normal instructions
     // -------------------------------------------------------------------------
@@ -124,12 +187,11 @@ public interface MethodVisitor {
      * 
      * @param opcode the opcode of the instruction to be visited. This opcode is
      *        either BIPUSH, SIPUSH or NEWARRAY.
-     * @param operand the operand of the instruction to be visited.<br>
-     *        When opcode is BIPUSH, operand value should be between
-     *        Byte.MIN_VALUE and Byte.MAX_VALUE.<br>
-     *        When opcode is SIPUSH, operand value should be between
-     *        Short.MIN_VALUE and Short.MAX_VALUE.<br>
-     *        When opcode is NEWARRAY, operand value should be one of
+     * @param operand the operand of the instruction to be visited.<br> When
+     *        opcode is BIPUSH, operand value should be between Byte.MIN_VALUE
+     *        and Byte.MAX_VALUE.<br> When opcode is SIPUSH, operand value
+     *        should be between Short.MIN_VALUE and Short.MAX_VALUE.<br> When
+     *        opcode is NEWARRAY, operand value should be one of
      *        {@link Opcodes#T_BOOLEAN}, {@link Opcodes#T_CHAR},
      *        {@link Opcodes#T_FLOAT}, {@link Opcodes#T_DOUBLE},
      *        {@link Opcodes#T_BYTE}, {@link Opcodes#T_SHORT},
@@ -151,15 +213,15 @@ public interface MethodVisitor {
 
     /**
      * Visits a type instruction. A type instruction is an instruction that
-     * takes a type descriptor as parameter.
+     * takes the internal name of a class as parameter.
      * 
      * @param opcode the opcode of the type instruction to be visited. This
      *        opcode is either NEW, ANEWARRAY, CHECKCAST or INSTANCEOF.
-     * @param desc the operand of the instruction to be visited. This operand is
-     *        must be a fully qualified class name in internal form, or the type
-     *        descriptor of an array type (see {@link Type Type}).
+     * @param type the operand of the instruction to be visited. This operand
+     *        must be the internal name of an object or array class (see {@link 
+     *        Type#getInternalName() getInternalName}).
      */
-    void visitTypeInsn(int opcode, String desc);
+    void visitTypeInsn(int opcode, String type);
 
     /**
      * Visits a field instruction. A field instruction is an instruction that
@@ -179,10 +241,11 @@ public interface MethodVisitor {
      * invokes a method.
      * 
      * @param opcode the opcode of the type instruction to be visited. This
-     *        opcode is either INVOKEVIRTUAL, INVOKESPECIAL, INVOKESTATIC or
-     *        INVOKEINTERFACE.
+     *        opcode is either INVOKEVIRTUAL, INVOKESPECIAL, INVOKESTATIC,
+     *        INVOKEINTERFACE or INVOKEDYNAMIC.
      * @param owner the internal name of the method's owner class (see {@link
-     *        Type#getInternalName() getInternalName}).
+     *        Type#getInternalName() getInternalName})
+     *        or {@link org.objectweb.asm.Opcodes#INVOKEDYNAMIC_OWNER}.
      * @param name the method's name.
      * @param desc the method's descriptor (see {@link Type Type}).
      */
@@ -242,7 +305,7 @@ public interface MethodVisitor {
      * @param labels beginnings of the handler blocks. <tt>labels[i]</tt> is
      *        the beginning of the handler block for the <tt>min + i</tt> key.
      */
-    void visitTableSwitchInsn(int min, int max, Label dflt, Label labels[]);
+    void visitTableSwitchInsn(int min, int max, Label dflt, Label[] labels);
 
     /**
      * Visits a LOOKUPSWITCH instruction.
@@ -252,7 +315,7 @@ public interface MethodVisitor {
      * @param labels beginnings of the handler blocks. <tt>labels[i]</tt> is
      *        the beginning of the handler block for the <tt>keys[i]</tt> key.
      */
-    void visitLookupSwitchInsn(Label dflt, int keys[], Label labels[]);
+    void visitLookupSwitchInsn(Label dflt, int[] keys, Label[] labels);
 
     /**
      * Visits a MULTIANEWARRAY instruction.
@@ -263,8 +326,7 @@ public interface MethodVisitor {
     void visitMultiANewArrayInsn(String desc, int dims);
 
     // -------------------------------------------------------------------------
-    // Exceptions table entries, debug information,
-    // max stack size and max locals
+    // Exceptions table entries, debug information, max stack and max locals
     // -------------------------------------------------------------------------
 
     /**
@@ -276,6 +338,9 @@ public interface MethodVisitor {
      * @param type internal name of the type of exceptions handled by the
      *        handler, or <tt>null</tt> to catch any exceptions (for "finally"
      *        blocks).
+     * @throws IllegalArgumentException if one of the labels has already been
+     *         visited by this visitor (by the {@link #visitLabel visitLabel}
+     *         method).
      */
     void visitTryCatchBlock(Label start, Label end, Label handler, String type);
 
