@@ -1,6 +1,6 @@
 /***
  * ASM: a very small and fast Java bytecode manipulation framework
- * Copyright (c) 2000-2005 INRIA, France Telecom
+ * Copyright (c) 2000-2007 INRIA, France Telecom
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,6 +29,8 @@
  */
 package org.objectweb.asm.tree;
 
+import java.util.Map;
+
 import org.objectweb.asm.MethodVisitor;
 import org.objectweb.asm.Opcodes;
 
@@ -42,7 +44,7 @@ public class LdcInsnNode extends AbstractInsnNode {
     /**
      * The constant to be loaded on the stack. This parameter must be a non null
      * {@link Integer}, a {@link Float}, a {@link Long}, a {@link Double}, a
-     * {@link String} or a {@link Type}.
+     * {@link String} or a {@link org.objectweb.asm.Type}.
      */
     public Object cst;
 
@@ -58,11 +60,15 @@ public class LdcInsnNode extends AbstractInsnNode {
         this.cst = cst;
     }
 
+    public int getType() {
+        return LDC_INSN;
+    }
+
     public void accept(final MethodVisitor mv) {
         mv.visitLdcInsn(cst);
     }
 
-    public int getType() {
-        return LDC_INSN;
+    public AbstractInsnNode clone(final Map labels) {
+        return new LdcInsnNode(cst);
     }
 }
\ No newline at end of file
