@@ -1,6 +1,6 @@
 /***
  * ASM: a very small and fast Java bytecode manipulation framework
- * Copyright (c) 2000,2002,2003 INRIA, France Telecom
+ * Copyright (c) 2000-2007 INRIA, France Telecom
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,76 +27,79 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
-
 package org.objectweb.asm.tree.analysis;
 
 import org.objectweb.asm.Type;
 
 /**
- * A {@link Value} that is represented by its type in a seven types type sytem.
+ * A {@link Value} that is represented by its type in a seven types type system.
  * This type system distinguishes the UNINITIALZED, INT, FLOAT, LONG, DOUBLE,
  * REFERENCE and RETURNADDRESS types.
  * 
  * @author Eric Bruneton
  */
-
 public class BasicValue implements Value {
 
-  public final static Value UNINITIALIZED_VALUE = new BasicValue(null);
-  
-  public final static Value INT_VALUE = new BasicValue(Type.INT_TYPE);
-  
-  public final static Value FLOAT_VALUE = new BasicValue(Type.FLOAT_TYPE);
-  
-  public final static Value LONG_VALUE = new BasicValue(Type.LONG_TYPE);
-  
-  public final static Value DOUBLE_VALUE = new BasicValue(Type.DOUBLE_TYPE);
-  
-  public final static Value REFERENCE_VALUE = new BasicValue(Type.getType("Ljava/lang/Object;"));
-  
-  public final static Value RETURNADDRESS_VALUE = new BasicValue(null);
-  
-  private Type type;
-  
-  public BasicValue (final Type type) {
-    this.type = type;
-  }
-  
-  public Type getType () {
-    return type;
-  }
-  
-  public int getSize () {
-    return type == Type.LONG_TYPE || type == Type.DOUBLE_TYPE ? 2 : 1;
-  }
-  
-  public boolean isReference () {
-    return type != null && (type.getSort() == Type.OBJECT || type.getSort() == Type.ARRAY);
-  }
-  
-  public boolean equals (final Value value) {
-    if (value == this) {
-      return true;
-    } else if (value instanceof BasicValue) {
-      if (type == null) {
-        return ((BasicValue)value).type == null; 
-      } else {
-        return type.equals(((BasicValue)value).type);
-      }
-    } else {
-      return false;
+    public static final Value UNINITIALIZED_VALUE = new BasicValue(null);
+
+    public static final Value INT_VALUE = new BasicValue(Type.INT_TYPE);
+
+    public static final Value FLOAT_VALUE = new BasicValue(Type.FLOAT_TYPE);
+
+    public static final Value LONG_VALUE = new BasicValue(Type.LONG_TYPE);
+
+    public static final Value DOUBLE_VALUE = new BasicValue(Type.DOUBLE_TYPE);
+
+    public static final Value REFERENCE_VALUE = new BasicValue(Type.getObjectType("java/lang/Object"));
+
+    public static final Value RETURNADDRESS_VALUE = new BasicValue(Type.VOID_TYPE);
+
+    private final Type type;
+
+    public BasicValue(final Type type) {
+        this.type = type;
     }
-  }
-  
-  public String toString () {
-    if (this == UNINITIALIZED_VALUE) {
-      return ".";
-    } else if (this == RETURNADDRESS_VALUE) {
-      return "A";
-    } else if (this == REFERENCE_VALUE) {
-      return "R";
-    } else {
-      return type.getDescriptor();
+
+    public Type getType() {
+        return type;
     }
-  }
-}
\ No newline at end of file
+
+    public int getSize() {
+        return type == Type.LONG_TYPE || type == Type.DOUBLE_TYPE ? 2 : 1;
+    }
+
+    public boolean isReference() {
+        return type != null
+                && (type.getSort() == Type.OBJECT || type.getSort() == Type.ARRAY);
+    }
+
+    public boolean equals(final Object value) {
+        if (value == this) {
+            return true;
+        } else if (value instanceof BasicValue) {
+            if (type == null) {
+                return ((BasicValue) value).type == null;
+            } else {
+                return type.equals(((BasicValue) value).type);
+            }
+        } else {
+            return false;
+        }
+    }
+
+    public int hashCode() {
+        return type == null ? 0 : type.hashCode();
+    }
+
+    public String toString() {
+        if (this == UNINITIALIZED_VALUE) {
+            return ".";
+        } else if (this == RETURNADDRESS_VALUE) {
+            return "A";
+        } else if (this == REFERENCE_VALUE) {
+            return "R";
+        } else {
+            return type.getDescriptor();
+        }
+    }
+}
