@@ -1,6 +1,6 @@
 /***
  * ASM: a very small and fast Java bytecode manipulation framework
- * Copyright (c) 2000-2005 INRIA, France Telecom
+ * Copyright (c) 2000-2007 INRIA, France Telecom
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,7 +38,6 @@ import org.objectweb.asm.ClassVisitor;
 import org.objectweb.asm.FieldVisitor;
 import org.objectweb.asm.MethodVisitor;
 import org.objectweb.asm.Opcodes;
-import org.objectweb.asm.Type;
 
 /**
  * A {@link ClassVisitor} that prints the ASM code that generates the classes it
@@ -52,25 +51,25 @@ import org.objectweb.asm.Type;
  * 
  * <pre>
  * import org.objectweb.asm.*;
- * 
+ *
  * public class HelloDump implements Opcodes {
- * 
+ *
  *     public static byte[] dump() throws Exception {
- * 
- *         ClassWriter cw = new ClassWriter(false);
+ *
+ *         ClassWriter cw = new ClassWriter(0);
  *         FieldVisitor fv;
  *         MethodVisitor mv;
  *         AnnotationVisitor av0;
- * 
+ *
  *         cw.visit(49,
  *                 ACC_PUBLIC + ACC_SUPER,
  *                 &quot;Hello&quot;,
  *                 null,
  *                 &quot;java/lang/Object&quot;,
  *                 null);
- * 
+ *
  *         cw.visitSource(&quot;Hello.java&quot;, null);
- * 
+ *
  *         {
  *             mv = cw.visitMethod(ACC_PUBLIC, &quot;&lt;init&gt;&quot;, &quot;()V&quot;, null, null);
  *             mv.visitVarInsn(ALOAD, 0);
@@ -102,18 +101,18 @@ import org.objectweb.asm.Type;
  *             mv.visitEnd();
  *         }
  *         cw.visitEnd();
- * 
+ *
  *         return cw.toByteArray();
  *     }
  * }
- * 
+ *
  * </pre>
  * 
  * </blockquote> where <tt>Hello</tt> is defined by: <p> <blockquote>
  * 
  * <pre>
  * public class Hello {
- * 
+ *
  *     public static void main(String[] args) {
  *         System.out.println(&quot;hello&quot;);
  *     }
@@ -132,12 +131,12 @@ public class ASMifierClassVisitor extends ASMifierAbstractVisitor implements
     /**
      * Pseudo access flag used to distinguish class access flags.
      */
-    private final static int ACCESS_CLASS = 262144;
+    private static final int ACCESS_CLASS = 262144;
 
     /**
      * Pseudo access flag used to distinguish field access flags.
      */
-    private final static int ACCESS_FIELD = 524288;
+    private static final int ACCESS_FIELD = 524288;
 
     /**
      * Pseudo access flag used to distinguish inner class flags.
@@ -151,7 +150,7 @@ public class ASMifierClassVisitor extends ASMifierAbstractVisitor implements
 
     /**
      * Prints the ASM source code to generate the given class to the standard
-     * output. <p> Usage: ASMifierClassVisitor [-debug] &lt;fully qualified
+     * output. <p> Usage: ASMifierClassVisitor [-debug] &lt;binary
      * class name or class file name&gt;
      * 
      * @param args the command line arguments.
@@ -161,15 +160,15 @@ public class ASMifierClassVisitor extends ASMifierAbstractVisitor implements
      */
     public static void main(final String[] args) throws Exception {
         int i = 0;
-        boolean skipDebug = true;
+        int flags = ClassReader.SKIP_DEBUG;
 
         boolean ok = true;
         if (args.length < 1 || args.length > 2) {
             ok = false;
         }
-        if (ok && args[0].equals("-debug")) {
+        if (ok && "-debug".equals(args[0])) {
             i = 1;
-            skipDebug = false;
+            flags = 0;
             if (args.length != 2) {
                 ok = false;
             }
@@ -178,17 +177,19 @@ public class ASMifierClassVisitor extends ASMifierAbstractVisitor implements
             System.err.println("Prints the ASM code to generate the given class.");
             System.err.println("Usage: ASMifierClassVisitor [-debug] "
                     + "<fully qualified class name or class file name>");
-            System.exit(-1);
+            return;
         }
         ClassReader cr;
-        if (args[i].endsWith(".class")) {
+        if (args[i].endsWith(".class") || args[i].indexOf('\\') > -1
+                || args[i].indexOf('/') > -1)
+        {
             cr = new ClassReader(new FileInputStream(args[i]));
         } else {
             cr = new ClassReader(args[i]);
         }
         cr.accept(new ASMifierClassVisitor(new PrintWriter(System.out)),
                 getDefaultAttributes(),
-                skipDebug);
+                flags);
     }
 
     /**
@@ -215,19 +216,19 @@ public class ASMifierClassVisitor extends ASMifierAbstractVisitor implements
     {
         String simpleName;
         int n = name.lastIndexOf('/');
-        if (n != -1) {
+        if (n == -1) {
+            simpleName = name;
+        } else {
             text.add("package asm." + name.substring(0, n).replace('/', '.')
                     + ";\n");
             simpleName = name.substring(n + 1);
-        } else {
-            simpleName = name;
         }
         text.add("import java.util.*;\n");
         text.add("import org.objectweb.asm.*;\n");
         text.add("import org.objectweb.asm.attrs.*;\n");
         text.add("public class " + simpleName + "Dump implements Opcodes {\n\n");
         text.add("public static byte[] dump () throws Exception {\n\n");
-        text.add("ClassWriter cw = new ClassWriter(false);\n");
+        text.add("ClassWriter cw = new ClassWriter(0);\n");
         text.add("FieldVisitor fv;\n");
         text.add("MethodVisitor mv;\n");
         text.add("AnnotationVisitor av0;\n\n");
@@ -382,12 +383,16 @@ public class ASMifierClassVisitor extends ASMifierAbstractVisitor implements
         }
         buf.append(");\n");
         text.add(buf.toString());
-        ASMifierMethodVisitor acv = new ASMifierMethodVisitor();
+        ASMifierMethodVisitor acv = createASMifierMethodVisitor();
         text.add(acv.getText());
         text.add("}\n");
         return acv;
     }
 
+    protected ASMifierMethodVisitor createASMifierMethodVisitor() {
+        return new ASMifierMethodVisitor();
+    }
+
     public AnnotationVisitor visitAnnotation(
         final String desc,
         final boolean visible)
@@ -432,16 +437,10 @@ public class ASMifierClassVisitor extends ASMifierAbstractVisitor implements
             first = false;
         }
         if ((access & Opcodes.ACC_PRIVATE) != 0) {
-            if (!first) {
-                buf.append(" + ");
-            }
             buf.append("ACC_PRIVATE");
             first = false;
         }
         if ((access & Opcodes.ACC_PROTECTED) != 0) {
-            if (!first) {
-                buf.append(" + ");
-            }
             buf.append("ACC_PROTECTED");
             first = false;
         }
@@ -463,10 +462,10 @@ public class ASMifierClassVisitor extends ASMifierAbstractVisitor implements
             if (!first) {
                 buf.append(" + ");
             }
-            if ((access & ACCESS_CLASS) != 0) {
-                buf.append("ACC_SUPER");
-            } else {
+            if ((access & ACCESS_CLASS) == 0) {
                 buf.append("ACC_SYNCHRONIZED");
+            } else {
+                buf.append("ACC_SUPER");
             }
             first = false;
         }
@@ -526,7 +525,7 @@ public class ASMifierClassVisitor extends ASMifierAbstractVisitor implements
             first = false;
         }
         if ((access & Opcodes.ACC_ANNOTATION) != 0
-                && ((access & ACCESS_CLASS) != 0))
+                && (access & ACCESS_CLASS) != 0)
         {
             if (!first) {
                 buf.append(" + ");
@@ -570,37 +569,7 @@ public class ASMifierClassVisitor extends ASMifierAbstractVisitor implements
             first = false;
         }
         if (first) {
-            buf.append("0");
-        }
-    }
-
-    /**
-     * Appends a string representation of the given constant to the given
-     * buffer.
-     * 
-     * @param buf a string buffer.
-     * @param cst an {@link java.lang.Integer Integer}, {@link java.lang.Float
-     *        Float}, {@link java.lang.Long Long},
-     *        {@link java.lang.Double Double} or {@link String String} object.
-     *        May be <tt>null</tt>.
-     */
-    static void appendConstant(final StringBuffer buf, final Object cst) {
-        if (cst == null) {
-            buf.append("null");
-        } else if (cst instanceof String) {
-            AbstractVisitor.appendString(buf, (String) cst);
-        } else if (cst instanceof Type) {
-            buf.append("Type.getType(\"")
-                    .append(((Type) cst).getDescriptor())
-                    .append("\")");
-        } else if (cst instanceof Integer) {
-            buf.append("new Integer(").append(cst).append(")");
-        } else if (cst instanceof Float) {
-            buf.append("new Float(\"").append(cst).append("\")");
-        } else if (cst instanceof Long) {
-            buf.append("new Long(").append(cst).append("L)");
-        } else if (cst instanceof Double) {
-            buf.append("new Double(\"").append(cst).append("\")");
+            buf.append('0');
         }
     }
 }
