@@ -97,26 +97,26 @@ public class Processor {
 
     private static final String SINGLE_XML_NAME = "classes.xml";
 
-    private int inRepresentation;
+    private final int inRepresentation;
 
-    private int outRepresentation;
+    private final int outRepresentation;
 
-    private InputStream input = null;
+    private final InputStream input;
 
-    private OutputStream output = null;
+    private final OutputStream output;
 
-    private Source xslt = null;
+    private final Source xslt;
 
-    private boolean computeMax;
+    private final boolean computeMax;
 
     private int n = 0;
 
     public Processor(
-        int inRepresenation,
-        int outRepresentation,
-        InputStream input,
-        OutputStream output,
-        Source xslt)
+        final int inRepresenation,
+        final int outRepresentation,
+        final InputStream input,
+        final OutputStream output,
+        final Source xslt)
     {
         this.inRepresentation = inRepresenation;
         this.outRepresentation = outRepresentation;
@@ -138,7 +138,9 @@ public class Processor {
         TransformerFactory tf = TransformerFactory.newInstance();
         if (!tf.getFeature(SAXSource.FEATURE)
                 || !tf.getFeature(SAXResult.FEATURE))
+        {
             return 0;
+        }
 
         SAXTransformerFactory saxtf = (SAXTransformerFactory) tf;
         Templates templates = null;
@@ -177,7 +179,7 @@ public class Processor {
 
         // configuring inputDocHandlerFactory
         // /////////////////////////////////////////////////
-        ContentHandler inDocHandler = null;
+        ContentHandler inDocHandler;
         if (templates == null) {
             inDocHandler = outDocHandler;
         } else {
@@ -198,7 +200,7 @@ public class Processor {
         }
 
         int i = 0;
-        ZipEntry ze = null;
+        ZipEntry ze;
         while ((ze = zis.getNextEntry()) != null) {
             update(ze.getName(), n++);
             if (isClassEntry(ze)) {
@@ -226,9 +228,12 @@ public class Processor {
         return i;
     }
 
-    private void copyEntry(InputStream is, OutputStream os) throws IOException {
-        if (outRepresentation == SINGLE_XML)
+    private void copyEntry(final InputStream is, final OutputStream os)
+            throws IOException
+    {
+        if (outRepresentation == SINGLE_XML) {
             return;
+        }
 
         byte[] buff = new byte[2048];
         int i;
@@ -237,7 +242,7 @@ public class Processor {
         }
     }
 
-    private boolean isClassEntry(ZipEntry ze) {
+    private boolean isClassEntry(final ZipEntry ze) {
         String name = ze.getName();
         return inRepresentation == SINGLE_XML && name.equals(SINGLE_XML_NAME)
                 || name.endsWith(".class") || name.endsWith(".class.xml");
@@ -245,8 +250,8 @@ public class Processor {
 
     private void processEntry(
         final ZipInputStream zis,
-        ZipEntry ze,
-        ContentHandlerFactory handlerFactory)
+        final ZipEntry ze,
+        final ContentHandlerFactory handlerFactory)
     {
         ContentHandler handler = handlerFactory.createContentHandler();
         try {
@@ -262,8 +267,7 @@ public class Processor {
             if (inRepresentation == BYTECODE) { // read bytecode and process it
                 // with handler
                 ClassReader cr = new ClassReader(readEntry(zis, ze));
-                cr.accept(new SAXClassAdapter(handler, singleInputDocument),
-                        false);
+                cr.accept(new SAXClassAdapter(handler, singleInputDocument), 0);
 
             } else { // read XML and process it with handler
                 XMLReader reader = XMLReaderFactory.createXMLReader();
@@ -279,7 +283,7 @@ public class Processor {
         }
     }
 
-    private EntryElement getEntryElement(ZipOutputStream zos) {
+    private EntryElement getEntryElement(final ZipOutputStream zos) {
         if (outRepresentation == SINGLE_XML) {
             return new SingleDocElement(zos);
         }
@@ -316,7 +320,7 @@ public class Processor {
     // return factory;
     // }
 
-    private String getName(ZipEntry ze) {
+    private String getName(final ZipEntry ze) {
         String name = ze.getName();
         if (isClassEntry(ze)) {
             if (inRepresentation != BYTECODE && outRepresentation == BYTECODE) {
@@ -325,7 +329,7 @@ public class Processor {
             } else if (inRepresentation == BYTECODE
                     && outRepresentation != BYTECODE)
             {
-                name = name.concat(".xml"); // .class to .class.xml
+                name += ".xml"; // .class to .class.xml
             }
             // } else if( CODE2ASM.equals( command)) {
             // name = name.substring( 0, name.length()-6).concat( ".asm");
@@ -333,17 +337,17 @@ public class Processor {
         return name;
     }
 
-    private byte[] readEntry(ZipInputStream zis, ZipEntry ze)
+    private static byte[] readEntry(final InputStream zis, final ZipEntry ze)
             throws IOException
     {
         long size = ze.getSize();
         if (size > -1) {
             byte[] buff = new byte[(int) size];
-            // int k = 0;
-            // while(( n = zis.read(buff, k, buff.length-k)) > 0) {
-            // k += n;
-            // }
-            zis.read(buff);
+            int k = 0;
+            int n;
+            while ((n = zis.read(buff, k, buff.length - k)) > 0) {
+                k += n;
+            }
             return buff;
         }
 
@@ -361,17 +365,17 @@ public class Processor {
      * 
      * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
      */
-    protected void update(Object arg, int n) {
+    protected void update(final Object arg, final int n) {
         if (arg instanceof Throwable) {
             ((Throwable) arg).printStackTrace();
         } else {
-            if ((n % 100) == 0) {
+            if (n % 100 == 0) {
                 System.err.println(n + " " + arg);
             }
         }
     }
 
-    public static void main(String[] args) throws Exception {
+    public static void main(final String[] args) throws Exception {
         if (args.length < 2) {
             showUsage();
             return;
@@ -380,8 +384,8 @@ public class Processor {
         int inRepresentation = getRepresentation(args[0]);
         int outRepresentation = getRepresentation(args[1]);
 
-        InputStream is = null;
-        OutputStream os = null;
+        InputStream is = System.in;
+        OutputStream os = new BufferedOutputStream(System.out);
 
         Source xslt = null;
         // boolean computeMax = true;
@@ -406,9 +410,7 @@ public class Processor {
             }
         }
 
-        if (is == null || os == null || inRepresentation == 0
-                || outRepresentation == 0)
-        {
+        if (inRepresentation == 0 || outRepresentation == 0) {
             showUsage();
             return;
         }
@@ -423,11 +425,11 @@ public class Processor {
         int n = m.process();
         long l2 = System.currentTimeMillis();
         System.err.println(n);
-        System.err.println("" + (l2 - l1) + "ms  " + (1000f * n / (l2 - l1))
+        System.err.println((l2 - l1) + "ms  " + 1000f * n / (l2 - l1)
                 + " resources/sec");
     }
 
-    private static int getRepresentation(String s) {
+    private static int getRepresentation(final String s) {
         if ("code".equals(s)) {
             return BYTECODE;
         } else if ("xml".equals(s)) {
@@ -439,7 +441,8 @@ public class Processor {
     }
 
     private static void showUsage() {
-        System.err.println("Usage: Main <in format> <out format> -in <input jar> -out <output jar> [-xslt <xslt fiel>]");
+        System.err.println("Usage: Main <in format> <out format> [-in <input jar>] [-out <output jar>] [-xslt <xslt fiel>]");
+        System.err.println("  when -in or -out is omitted sysin and sysout would be used");
         System.err.println("  <in format> and <out format> - code | xml | singlexml");
     }
 
@@ -450,8 +453,7 @@ public class Processor {
     private static final class ProtectedInputStream extends InputStream {
         private final InputStream is;
 
-        private ProtectedInputStream(InputStream is) {
-            super();
+        ProtectedInputStream(final InputStream is) {
             this.is = is;
         }
 
@@ -462,7 +464,9 @@ public class Processor {
             return is.read();
         }
 
-        public final int read(byte[] b, int off, int len) throws IOException {
+        public final int read(final byte[] b, final int off, final int len)
+                throws IOException
+        {
             return is.read(b, off, len);
         }
 
@@ -493,11 +497,14 @@ public class Processor {
     private static final class SAXWriterFactory implements
             ContentHandlerFactory
     {
-        private Writer w;
+        private final Writer w;
 
-        private boolean optimizeEmptyElements;
+        private final boolean optimizeEmptyElements;
 
-        public SAXWriterFactory(Writer w, boolean optimizeEmptyElements) {
+        SAXWriterFactory(
+            final Writer w,
+            final boolean optimizeEmptyElements)
+        {
             this.w = w;
             this.optimizeEmptyElements = optimizeEmptyElements;
         }
@@ -516,9 +523,12 @@ public class Processor {
     {
         private OutputStream os;
 
-        private boolean computeMax;
+        private final boolean computeMax;
 
-        public ASMContentHandlerFactory(OutputStream os, boolean computeMax) {
+        ASMContentHandlerFactory(
+            final OutputStream os,
+            final boolean computeMax)
+        {
             this.os = os;
             this.computeMax = computeMax;
         }
@@ -537,14 +547,14 @@ public class Processor {
     {
         private SAXTransformerFactory saxtf;
 
-        private Templates templates;
+        private final Templates templates;
 
         private ContentHandler outputHandler;
 
-        public TransformerHandlerFactory(
-            SAXTransformerFactory saxtf,
-            Templates templates,
-            ContentHandler outputHandler)
+        TransformerHandlerFactory(
+            final SAXTransformerFactory saxtf,
+            final Templates templates,
+            final ContentHandler outputHandler)
         {
             this.saxtf = saxtf;
             this.templates = templates;
@@ -565,12 +575,13 @@ public class Processor {
     /**
      * SubdocumentHandlerFactory
      */
-    private final static class SubdocumentHandlerFactory implements
+    private static final class SubdocumentHandlerFactory implements
             ContentHandlerFactory
     {
-        private ContentHandler subdocumentHandler;
+        private final ContentHandler subdocumentHandler;
 
-        public SubdocumentHandlerFactory(ContentHandler subdocumentHandler) {
+        SubdocumentHandlerFactory(final ContentHandler subdocumentHandler)
+        {
             this.subdocumentHandler = subdocumentHandler;
         }
 
@@ -588,14 +599,14 @@ public class Processor {
      * <i><blockquote> This implementation does not support namespaces, entity
      * definitions (uncluding DTD), CDATA and text elements. </blockquote></i>
      */
-    private final static class SAXWriter extends DefaultHandler implements
+    private static final class SAXWriter extends DefaultHandler implements
             LexicalHandler
     {
         private static final char[] OFF = "                                                                                                        ".toCharArray();
 
         private Writer w;
 
-        private boolean optimizeEmptyElements;
+        private final boolean optimizeEmptyElements;
 
         private boolean openElement = false;
 
@@ -608,29 +619,30 @@ public class Processor {
          * @param optimizeEmptyElements if set to <code>true</code>, short
          *        XML syntax will be used for empty elements
          */
-        public SAXWriter(Writer w, boolean optimizeEmptyElements) {
+        SAXWriter(final Writer w, final boolean optimizeEmptyElements) {
             this.w = w;
             this.optimizeEmptyElements = optimizeEmptyElements;
         }
 
         public final void startElement(
-            String ns,
-            String localName,
-            String qName,
-            Attributes atts) throws SAXException
+            final String ns,
+            final String localName,
+            final String qName,
+            final Attributes atts) throws SAXException
         {
             try {
                 closeElement();
 
                 writeIdent();
-                w.write("<".concat(qName));
-                if (atts != null && atts.getLength() > 0)
+                w.write('<' + qName);
+                if (atts != null && atts.getLength() > 0) {
                     writeAttributes(atts);
+                }
 
-                if (!optimizeEmptyElements) {
-                    w.write(">\n");
-                } else {
+                if (optimizeEmptyElements) {
                     openElement = true;
+                } else {
+                    w.write(">\n");
                 }
                 ident += 2;
 
@@ -640,8 +652,10 @@ public class Processor {
             }
         }
 
-        public final void endElement(String ns, String localName, String qName)
-                throws SAXException
+        public final void endElement(
+            final String ns,
+            final String localName,
+            final String qName) throws SAXException
         {
             ident -= 2;
             try {
@@ -669,7 +683,7 @@ public class Processor {
             }
         }
 
-        public final void comment(char[] ch, int off, int len)
+        public final void comment(final char[] ch, final int off, final int len)
                 throws SAXException
         {
             try {
@@ -686,18 +700,20 @@ public class Processor {
             }
         }
 
-        public final void startDTD(String arg0, String arg1, String arg2)
-                throws SAXException
+        public final void startDTD(
+            final String arg0,
+            final String arg1,
+            final String arg2) throws SAXException
         {
         }
 
         public final void endDTD() throws SAXException {
         }
 
-        public final void startEntity(String arg0) throws SAXException {
+        public final void startEntity(final String arg0) throws SAXException {
         }
 
-        public final void endEntity(String arg0) throws SAXException {
+        public final void endEntity(final String arg0) throws SAXException {
         }
 
         public final void startCDATA() throws SAXException {
@@ -706,15 +722,17 @@ public class Processor {
         public final void endCDATA() throws SAXException {
         }
 
-        private final void writeAttributes(Attributes atts) throws IOException {
+        private final void writeAttributes(final Attributes atts)
+                throws IOException
+        {
             StringBuffer sb = new StringBuffer();
             int len = atts.getLength();
             for (int i = 0; i < len; i++) {
-                sb.append(" ")
+                sb.append(' ')
                         .append(atts.getLocalName(i))
                         .append("=\"")
                         .append(esc(atts.getValue(i)))
-                        .append("\"");
+                        .append('\"');
             }
             w.write(sb.toString());
         }
@@ -725,7 +743,7 @@ public class Processor {
          * @param str string to encode.
          * @return encoded string
          */
-        private final String esc(String str) {
+        private static final String esc(final String str) {
             StringBuffer sb = new StringBuffer(str.length());
             for (int i = 0; i < str.length(); i++) {
                 char ch = str.charAt(i);
@@ -791,10 +809,10 @@ public class Processor {
      * hardly fit into the memory all together. <p> TODO use complete path for
      * subdocumentRoot
      */
-    private final static class InputSlicingHandler extends DefaultHandler {
+    private static final class InputSlicingHandler extends DefaultHandler {
         private String subdocumentRoot;
 
-        private ContentHandler rootHandler;
+        private final ContentHandler rootHandler;
 
         private ContentHandlerFactory subdocumentHandlerFactory;
 
@@ -815,10 +833,10 @@ public class Processor {
          *        create {@link ContentHandler ContentHandler} instances for
          *        subdocuments.
          */
-        public InputSlicingHandler(
-            String subdocumentRoot,
-            ContentHandler rootHandler,
-            ContentHandlerFactory subdocumentHandlerFactory)
+        InputSlicingHandler(
+            final String subdocumentRoot,
+            final ContentHandler rootHandler,
+            final ContentHandlerFactory subdocumentHandlerFactory)
         {
             this.subdocumentRoot = subdocumentRoot;
             this.rootHandler = rootHandler;
@@ -826,10 +844,10 @@ public class Processor {
         }
 
         public final void startElement(
-            String namespaceURI,
-            String localName,
-            String qName,
-            Attributes list) throws SAXException
+            final String namespaceURI,
+            final String localName,
+            final String qName,
+            final Attributes list) throws SAXException
         {
             if (subdocument) {
                 subdocumentHandler.startElement(namespaceURI,
@@ -850,9 +868,9 @@ public class Processor {
         }
 
         public final void endElement(
-            String namespaceURI,
-            String localName,
-            String qName) throws SAXException
+            final String namespaceURI,
+            final String localName,
+            final String qName) throws SAXException
         {
             if (subdocument) {
                 subdocumentHandler.endElement(namespaceURI, localName, qName);
@@ -878,8 +896,10 @@ public class Processor {
             }
         }
 
-        public final void characters(char[] buff, int offset, int size)
-                throws SAXException
+        public final void characters(
+            final char[] buff,
+            final int offset,
+            final int size) throws SAXException
         {
             if (subdocument) {
                 subdocumentHandler.characters(buff, offset, size);
@@ -896,15 +916,16 @@ public class Processor {
      * {@link org.xml.sax.ContentHandler ContentHandler} obtained from
      * {@link java.net.ContentHandlerFactory ContentHandlerFactory}. This is
      * useful for running XSLT engine against large XML document that will
-     * hardly fit into the memory all together. <p> TODO use complete path for
-     * subdocumentRoot
+     * hardly fit into the memory all together.
+     * 
+     * <p> TODO use complete path for subdocumentRoot
      */
     private static final class OutputSlicingHandler extends DefaultHandler {
-        private String subdocumentRoot;
+        private final String subdocumentRoot;
 
         private ContentHandlerFactory subdocumentHandlerFactory;
 
-        private EntryElement entryElement;
+        private final EntryElement entryElement;
 
         private boolean isXml;
 
@@ -916,19 +937,17 @@ public class Processor {
          * Constructs a new {@link OutputSlicingHandler SubdocumentHandler}
          * object.
          * 
-         * @param subdocumentRoot name/path to the root element of the
-         *        subdocument
-         * @param rootHandler content handler for the entire document
-         *        (subdocument envelope).
          * @param subdocumentHandlerFactory a
          *        {@link ContentHandlerFactory ContentHandlerFactory} used to
          *        create {@link ContentHandler ContentHandler} instances for
          *        subdocuments.
+         * @param entryElement TODO.
+         * @param isXml TODO.
          */
-        public OutputSlicingHandler(
-            ContentHandlerFactory subdocumentHandlerFactory,
-            EntryElement entryElement,
-            boolean isXml)
+        OutputSlicingHandler(
+            final ContentHandlerFactory subdocumentHandlerFactory,
+            final EntryElement entryElement,
+            final boolean isXml)
         {
             this.subdocumentRoot = "class";
             this.subdocumentHandlerFactory = subdocumentHandlerFactory;
@@ -937,10 +956,10 @@ public class Processor {
         }
 
         public final void startElement(
-            String namespaceURI,
-            String localName,
-            String qName,
-            Attributes list) throws SAXException
+            final String namespaceURI,
+            final String localName,
+            final String qName,
+            final Attributes list) throws SAXException
         {
             if (subdocument) {
                 subdocumentHandler.startElement(namespaceURI,
@@ -949,12 +968,13 @@ public class Processor {
                         list);
             } else if (localName.equals(subdocumentRoot)) {
                 String name = list.getValue("name");
-                if (name == null || name.length() == 0)
+                if (name == null || name.length() == 0) {
                     throw new SAXException("Class element without name attribute.");
+                }
                 try {
                     entryElement.openEntry(isXml
-                            ? name.concat(".class.xml")
-                            : name.concat(".class"));
+                            ? name + ".class.xml"
+                            : name + ".class");
                 } catch (IOException ex) {
                     throw new SAXException(ex.toString(), ex);
                 }
@@ -969,9 +989,9 @@ public class Processor {
         }
 
         public final void endElement(
-            String namespaceURI,
-            String localName,
-            String qName) throws SAXException
+            final String namespaceURI,
+            final String localName,
+            final String qName) throws SAXException
         {
             if (subdocument) {
                 subdocumentHandler.endElement(namespaceURI, localName, qName);
@@ -993,8 +1013,10 @@ public class Processor {
         public final void endDocument() throws SAXException {
         }
 
-        public final void characters(char[] buff, int offset, int size)
-                throws SAXException
+        public final void characters(
+            final char[] buff,
+            final int offset,
+            final int size) throws SAXException
         {
             if (subdocument) {
                 subdocumentHandler.characters(buff, offset, size);
@@ -1012,13 +1034,13 @@ public class Processor {
     }
 
     private static final class SingleDocElement implements EntryElement {
-        private OutputStream os;
+        private final OutputStream os;
 
-        public SingleDocElement(OutputStream os) {
+        SingleDocElement(final OutputStream os) {
             this.os = os;
         }
 
-        public OutputStream openEntry(String name) throws IOException {
+        public OutputStream openEntry(final String name) throws IOException {
             return os;
         }
 
@@ -1031,11 +1053,11 @@ public class Processor {
     private static final class ZipEntryElement implements EntryElement {
         private ZipOutputStream zos;
 
-        public ZipEntryElement(ZipOutputStream zos) {
+        ZipEntryElement(final ZipOutputStream zos) {
             this.zos = zos;
         }
 
-        public OutputStream openEntry(String name) throws IOException {
+        public OutputStream openEntry(final String name) throws IOException {
             ZipEntry entry = new ZipEntry(name);
             zos.putNextEntry(entry);
             return zos;
